@@ -291,16 +291,25 @@ class ReverseWrapper(object):
 
         sink.indent()
 
-        if len(self.pyret_parse_items) == 1:
-            ## if retval is one item only, pack it in a tuple so we
-            ## can use PyArg_ParseTuple as usual..
-            self.write_code('py_retval = Py_BuildValue("(N)", py_retval);')
-        if len(self.pyret_parse_items) > 0:
-            ## Parse return values using PyArg_ParseTuple
-            self.write_code(code=None, failure_expression=(
-                '!PyArg_ParseTuple(py_retval, "%s", %s)' % (
-                "".join([format for format, param in self.pyret_parse_items]),
-                ", ".join([param for format, param in self.pyret_parse_items]))))
+        if self.pyret_parse_items == [("", "")]:
+            ## special case when there are no return parameters
+            self.write_code(
+                code=None,
+                failure_expression='py_retval != Py_None',
+                failure_exception=('PyErr_SetString(PyExc_TypeError, '
+                                   '"virtual method should return None");'))
+        else:    
+            if len(self.pyret_parse_items) == 1:
+                ## if retval is one item only, pack it in a tuple so we
+                ## can use PyArg_ParseTuple as usual..
+                self.write_code('py_retval = Py_BuildValue("(N)", py_retval);')
+            if len(self.pyret_parse_items) > 0:
+                ## Parse return values using PyArg_ParseTuple
+                params = ["py_retval",
+                          '"%s"' % "".join([format for format, param in self.pyret_parse_items])]
+                params.extend([param for format, param in self.pyret_parse_items if param])
+                self.write_code(code=None, failure_expression=(
+                    '!PyArg_ParseTuple(%s)' % (', '.join(params),)))
 
         if DEBUG_MODE:
             self.declarations.writeln("/* end declarations */")
@@ -425,10 +434,7 @@ class VoidReturn(ReturnType):
         self.wrapper.write_code("return;")
 
     def write_conversion(self):
-        self.wrapper.write_code(
-            code=None,
-            failure_expression="py_retval != Py_None",
-            failure_exception='PyErr_SetString(PyExc_TypeError, "retval should be None");')
+        self.wrapper.add_pyret_parse_item("", "", prepend=True)
 
 argtypes.matcher.register_reverse_ret('void', VoidReturn)
 argtypes.matcher.register_reverse_ret('none', VoidReturn)
@@ -667,12 +673,7 @@ class DoubleReturn(ReturnType):
     def write_error_return(self):
         self.wrapper.write_code("return -G_MAXFLOAT;")
     def write_conversion(self):
-        self.wrapper.write_code(
-            code=None,
-            failure_expression="!PyFloat_AsDouble(py_retval)",
-            failure_exception=
-            'PyErr_SetString(PyExc_TypeError, "retval should be a float");')
-        self.wrapper.write_code("retval = PyFloat_AsDouble(py_retval);")
+        self.wrapper.add_pyret_parse_item("d", "&retval", prepend=True)
 
 for argtype in ('float', 'double', 'gfloat', 'gdouble'):
     argtypes.matcher.register_reverse(argtype, DoubleParam)
